/**
 * EasyBeans
 * Copyright (C) 2006 Bull S.A.S.
 * Contact: easybeans@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: JoramService.java 695 2006-06-20 20:54:57Z benoitf $
 * --------------------------------------------------------------------------
 */

package org.objectweb.easybeans.jms.joram;

import java.net.ConnectException;
import java.util.ArrayList;
import java.util.List;
import java.util.Properties;

import javax.jms.ConnectionFactory;
import javax.jms.QueueConnectionFactory;
import javax.jms.TopicConnectionFactory;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import javax.resource.ResourceException;
import javax.resource.spi.ActivationSpec;
import javax.resource.spi.BootstrapContext;
import javax.resource.spi.ResourceAdapter;
import javax.resource.spi.ResourceAdapterInternalException;
import javax.resource.spi.XATerminator;
import javax.resource.spi.work.WorkManager;
import javax.transaction.TransactionManager;
import javax.transaction.xa.XAException;

import org.objectweb.easybeans.jca.workmanager.ResourceWorkManager;
import org.objectweb.easybeans.transaction.JTransactionManager;
import org.objectweb.joram.client.connector.ActivationSpecImpl;
import org.objectweb.joram.client.connector.AdapterTracing;
import org.objectweb.joram.client.connector.JoramAdapter;
import org.objectweb.joram.client.jms.admin.AdminException;
import org.objectweb.joram.client.jms.admin.AdminModule;
import org.objectweb.joram.client.jms.tcp.QueueTcpConnectionFactory;
import org.objectweb.joram.client.jms.tcp.TcpConnectionFactory;
import org.objectweb.joram.client.jms.tcp.TopicTcpConnectionFactory;
import org.objectweb.joram.mom.proxies.ConnectionManager;
import org.objectweb.joram.mom.proxies.tcp.TcpProxyService;
import org.objectweb.jotm.Current;
import org.objectweb.util.monolog.Monolog;
import org.objectweb.util.monolog.api.BasicLevel;

import fr.dyade.aaa.agent.AgentServer;
import fr.dyade.aaa.agent.ServerConfigHelper;
import fr.dyade.aaa.util.NullTransaction;

/**
 * Class that start/stop a simple collocated JORAM server.
 * It also creates some initial Topics/Queues.
 * @author Florent Benoit
 */
public class JoramService {

    /**
     * Queue connection factory (for external components).
     */
    private static final String QUEUE_CONN_FACT_NAME="JQCF";

    /**
     * Topic connection factory (for external components).
     */
    private static final String TOPIC_CONN_FACT_NAME="JTCF";

    /**
     * Connection factory (for external components).
     */
    private static final String CONN_FACT_NAME="JCF";

    /**
     * Default port number.
     */
    private static final int DEFAULT_PORT_NUMBER = 16010;

    /**
     * ID of the JORAM server.
     */
    private static final short ID = 0;

    /**
     * Transaction property (Set to remove persistence settings : transient). It
     * avoids the creation of a directory.
     */
    private static final String TRANSACTION_PROPERTY = "Transaction";

    /**
     * Default name of the persistence directory (won't be used).
     */
    private static final String DEFAULT_PERSISTENCE_DIRECTORY = "joram-persistence-s" + ID;

    /**
     * Server is started ?
     */
    private boolean started = false;

    /**
     * Minimum threads.
     */
    private static final int MIN_THREADS = 1;

    /**
     * Maximum threads.
     */
    private static final int MAX_THREADS = 2;

    /**
     * Thread timeout.
     */
    private static final int THREAD_TIMEOUT = 20;

    /**
     * Instance of the resource adapter.
     */
    private JoramAdapter joramAdapter = null;

    /**
     * List of topics to create.
     */
    private List<String> topics = null;

    /**
     * List of queues to create.
     */
    private List<String> queues = null;


    /**
     * Initial Context.
     */
    private InitialContext ictx = null;

    /**
     * Default constructor.
     */
    public JoramService() {
        topics = new ArrayList<String>();
        queues = new ArrayList<String>();
    }

    /**
     * Starts a Joram Server without persistence.
     * @throws JoramException if start fails
     */
    public void start() throws JoramException {

        // Initialize Monolog
        Properties properties = new Properties();
        properties.put(Monolog.MONOLOG_CLASS_NAME, "org.objectweb.util.monolog.wrapper.javaLog.LoggerFactory");
        properties.put("logger.org.objectweb.joram.level", "ERROR");
        properties.put("logger.com.scalagent.level", "ERROR");
        properties.put("logger.fr.dyade.aaa.level", "ERROR");
        Monolog.init(properties);


        // Build initial context
        try {
            ictx = new InitialContext();
        } catch (NamingException e) {
            throw new JoramException("Cannot create an initial context.", e);
        }


        TransactionManager transactionManager = JTransactionManager.getTransactionManager();
        WorkManager workManager = new ResourceWorkManager(transactionManager, MIN_THREADS, MAX_THREADS, THREAD_TIMEOUT);

        // Create BootstrapContext
        XATerminator xaTerminator = null;
        try {
            xaTerminator = ((Current) transactionManager).getXATerminator();
        } catch (XAException e) {
            throw new JoramException("Cannot get the XA terminator", e);
        }

        BootstrapContext bootstrapContext = new JoramBootstrapContext(workManager, xaTerminator);

        // Create JORAM adapter
        joramAdapter = new JoramAdapter();

        // configure it (in fact the server will be collocated but started by this service and not by the resource adapter).
        joramAdapter.setCollocatedServer(Boolean.FALSE);

        // Set properties (transient)
        System.setProperty(TRANSACTION_PROPERTY, NullTransaction.class.getName());

        // Init
        try {
            AgentServer.init(ID, DEFAULT_PERSISTENCE_DIRECTORY, null);
        } catch (Exception e) {
            throw new JoramException("Cannot initialize a new collocated Joram server", e);
        }


        // start Agent
       try {
           AgentServer.start();
        } catch (Exception e) {
            throw new JoramException("Cannot start collocated Joram server", e);
        }

        // Add services
        try {
            new ServerConfigHelper(false).addService(ID, ConnectionManager.class.getName(), "root root");
        } catch (Exception e) {
            throw new JoramException("Cannot add connection manager service", e);
        }
        // To enable TCP listener
        try {
            new ServerConfigHelper(false).addService(ID, TcpProxyService.class.getName(), "16010");
        } catch (Exception e) {
            throw new JoramException("Cannot add TcpProxy service", e);
        }



        // connect to the collocated server
        connectToCollocated();


        // Create anonymous user
        try {
            joramAdapter.createUser("anonymous", "anonymous");
        } catch (AdminException e) {
            throw new JoramException("Cannot create anonymous user", e);

        }


        // start resource adapter
        try {
            joramAdapter.start(bootstrapContext);
        } catch (ResourceAdapterInternalException e) {
            throw new JoramException("Cannot start the resource adapter of JORAM", e);
        }

        // create factories
        createConnectionFactories();


        // initial topics and queues
        createInitialTopics();
        createInitialQueues();

        // Create and bind ActivationSpec
        ActivationSpec activationSpec = new ActivationSpecImpl();
        try {
            activationSpec.setResourceAdapter(joramAdapter);
        } catch (ResourceException e) {
            throw new JoramException("Cannot set resource adapter on activation spec object", e);
        }
        try {
            ictx.rebind("joramActivationSpec", activationSpec);
        } catch (NamingException e) {
            throw new JoramException("Cannot bind activation spec object", e);
        }


        started = true;
    }

    /**
     * Stops the JORAM server (if started).
     * @throws JoramException if stop is failing
     */
    public void stop() throws JoramException {
        if (!started) {
            throw new IllegalStateException("Cannot stop a server as it was not started");
        }

        // Stop adapter
        joramAdapter.stop();

        // disconnect
        disconnectFromCollocated();

        // stop
        AgentServer.stop();
    }

    /**
     * Connect to the collocated server to performg administration tasks. It
     * needs to be called before any admin task.
     * @throws JoramException if the connection to the collocated server fails
     */
    private void connectToCollocated() throws JoramException {
        try {
            AdminModule.collocatedConnect("root", "root");
        } catch (ConnectException e) {
            throw new JoramException("Cannot connect to the collocated server for the administration", e);
        } catch (AdminException e) {
            throw new JoramException("Cannot connect to the collocated server for the administration", e);
        }
    }

    /**
     * Disconnect from the collocated server. It needs to be called when
     * stopping to use admin task.
     * @throws AdminException
     * @throws ConnectException
     */
    private void disconnectFromCollocated() {
        AdminModule.disconnect();
    }

    /**
     * Create connection factories.
     * @throws JoramException if factories are not created.
     */
    private void createConnectionFactories() throws JoramException {
        // managed
        joramAdapter.createCF("CF");
        joramAdapter.createQCF("QCF");
        joramAdapter.createTCF("TCF");

        // Create connection factories  that will be used by a pure JMS Client
        ConnectionFactory jcf = null;
        TopicConnectionFactory jtcf = null;
        QueueConnectionFactory jqcf = null;

        String host = "localhost";
        int port = DEFAULT_PORT_NUMBER;
        String name = CONN_FACT_NAME;
        try {
            jcf = TcpConnectionFactory.create(host, port);
            ictx.rebind(name, jcf);
        } catch (NamingException e) {
            throw new JoramException("Cannot create a factory with the name '" + name + "'.", e);
        }

        name = QUEUE_CONN_FACT_NAME;
        try {
            jqcf = QueueTcpConnectionFactory.create(host, port);
            ictx.rebind(name, jqcf);
        } catch (NamingException e) {
            throw new JoramException("Cannot create a factory with the name '" + name + "'.", e);
        }

        name = TOPIC_CONN_FACT_NAME;
        try {
            jtcf = TopicTcpConnectionFactory.create(host, port);
            ictx.rebind(name, jtcf);
        } catch (NamingException e) {
            throw new JoramException("Cannot create a factory with the name '" + name + "'.", e);
        }
    }

    /**
     * Creates a topic with a given name.
     * @param name the topic's name.
     * @throws JoramException if the topic can't be created
     */
    private void createTopic(final String name) throws JoramException {
        try {
            joramAdapter.createTopic(name);
        } catch (AdminException e) {
            throw new JoramException("Cannot create a topic with the name '" + name + "'.", e);
        }
    }

    /**
     * Creates a queue with a given name.
     * @param name the topic's name.
     * @throws JoramException if the queue can't be created
     */
    private void createQueue(final String name) throws JoramException {
        try {
            joramAdapter.createQueue(name);
        } catch (AdminException e) {
            throw new JoramException("Cannot create a queue with the name '" + name + "'.", e);
        }
    }

    /**
     * Create the list of the defined topics.
     * @throws JoramException if topics can't be created
     */
    private void createInitialTopics() throws JoramException {
        for (String topic : topics) {
            createTopic(topic);
        }
    }

    /**
     * Create the list of the defined queues.
     * @throws JoramException if queues can't be created
     */
    private void createInitialQueues() throws JoramException {
        for (String queue : queues) {
            createQueue(queue);
        }
    }

    /**
     * Sets the initial queues of the Joram server.
     * @param queues the list of the name of the queues.
     */
    public void setQueues(final List<String> queues) {
        this.queues = queues;
    }

    /**
     * Sets the initial topics of the Joram server.
     * @param topics the list of the name of the topics.
     */
    public void setTopics(final List<String> topics) {
        this.topics = topics;
    }


    /**
     * Gets the resource adapter instance.
     * @return resource adapter instance.
     */
    public ResourceAdapter getResourceAdapter() {
        return joramAdapter;
    }

}
