/**
 * EasyBeans
 * Copyright (C) 2006 Bull S.A.S.
 * Contact: easybeans@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: Deployment.java 503 2006-05-24 13:44:31Z benoitf $
 * --------------------------------------------------------------------------
 */

package org.objectweb.easybeans.deployment;

import java.io.File;

import org.objectweb.easybeans.deployment.annotations.analyzer.AnnotationDeploymentAnalyzer;
import org.objectweb.easybeans.deployment.annotations.exceptions.AnalyzerException;
import org.objectweb.easybeans.deployment.annotations.exceptions.ResolverException;
import org.objectweb.easybeans.deployment.annotations.helper.ResolverHelper;
import org.objectweb.easybeans.deployment.annotations.metadata.ClassAnnotationMetadata;
import org.objectweb.easybeans.deployment.xml.EJB3DeploymentDesc;
import org.objectweb.easybeans.deployment.xml.EJB3DeploymentDescException;
import org.objectweb.easybeans.deployment.xml.helper.MetadataMerge;
import org.objectweb.easybeans.deployment.xml.struct.EJB3;
import org.objectweb.easybeans.log.JLog;
import org.objectweb.easybeans.log.JLogFactory;

/**
 * This class will parse given ejb-jar file and completes metadata by using
 * resolver.
 * @author Florent Benoit
 */
public class Deployment {

    /**
     * Logger.
     */
    private static JLog logger = JLogFactory.getLog(Deployment.class);

    /**
     * Archive which will be analyzed.
     */
    private File archive = null;

    /**
     * Annotation deployment analyzer.
     */
    private AnnotationDeploymentAnalyzer annotationDeploymentAnalyzer = null;

    /**
     * Constructor.<br> Archive which will be used when analyzing.
     * @param archive the archive to analyze.
     */
    public Deployment(final File archive) {
        this.archive = archive;
        this.annotationDeploymentAnalyzer = new AnnotationDeploymentAnalyzer(archive);
    }

    /**
     * Analyzes the jarFile.
     * @throws AnalyzerException if analyze of jar file fails.
     * @throws ResolverException if resolver fails.
     * @throws EJB3DeploymentDescException if parsing fails.
     */
    public void analyze() throws AnalyzerException, EJB3DeploymentDescException, ResolverException {

        // for time debugging
        long tAnalyzeStart = System.currentTimeMillis();
        annotationDeploymentAnalyzer.analyze();
        // time if debugging
        if (logger.isDebugEnabled()) {
            long tAnalyzeStartEnd = System.currentTimeMillis();
            logger.debug("Analyze of file '" + archive.getName() + "' took '" + (tAnalyzeStartEnd - tAnalyzeStart)
                    + "' ms.");
        }

        // Get EJB3 DD.
        EJB3 ejb3 = EJB3DeploymentDesc.getEjb3(archive);
        annotationDeploymentAnalyzer.getEjbJarAnnotationMetadata().setEjb3(ejb3);

        // Merge DD with annotations metadata
        MetadataMerge.merge(annotationDeploymentAnalyzer.getEjbJarAnnotationMetadata());

        // Complete metadata
        long tResolverStart = System.currentTimeMillis();
        ResolverHelper.resolve(annotationDeploymentAnalyzer.getEjbJarAnnotationMetadata());
        // time if debugging
        if (logger.isDebugEnabled()) {
            long tResolverEnd = System.currentTimeMillis();
            logger.debug("Resolver on metadata from '" + archive.getName() + "' took '"
                    + (tResolverEnd - tResolverStart) + "' ms.");
        }

        if (logger.isDebugEnabled()) {
            for (ClassAnnotationMetadata classAnnotationMetadata : annotationDeploymentAnalyzer
                    .getEjbJarAnnotationMetadata().getClassAnnotationMetadataCollection()) {
                logger.debug("Result for class = " + classAnnotationMetadata);
            }
        }
    }

    /**
     * @return annotation deployment analyzer
     */
    public AnnotationDeploymentAnalyzer getAnnotationDeploymentAnalyzer() {
        return annotationDeploymentAnalyzer;
    }
}
