/**
 * EasyBeans
 * Copyright (C) 2006 Bull S.A.S.
 * Contact: easybeans@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: JPersistenceUnitInfoHelper.java 463 2006-05-17 15:00:58Z benoitf $
 * --------------------------------------------------------------------------
 */

package org.objectweb.easybeans.persistence.xml;

import java.io.IOException;
import java.net.URL;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.Map;
import java.util.Properties;

import javax.naming.InitialContext;
import javax.naming.NamingException;
import javax.sql.DataSource;

import org.objectweb.easybeans.log.JLog;
import org.objectweb.easybeans.log.JLogFactory;

/**
 * Class used to fill PersistenceInfo implementation class.
 * @author Florent Benoit
 */
public final class JPersistenceUnitInfoHelper {

    /**
     * Default XML configuration.
     */
    private static final String DEFAULT_PERSISTENCE_CONFIG = "org/objectweb/easybeans/persistence/conf/default-persistence.xml";

    /**
     * Logger.
     */
    private static JLog logger = JLogFactory.getLog(JPersistenceUnitInfoHelper.class);

    /**
     * Default configuration of persistence.xml (it defines the default
     * persistence provider and links to external config, one for each provider
     * class name.
     */
    private static JPersistenceUnitInfo defaultPersistenceunitInfo = null;

    /**
     * Defines the default configuration (properties) for each persistence
     * provider.
     */
    private static Map<String, JPersistenceUnitInfo> providersInfo = null;

    /**
     * Utility class, no public constructor.
     */
    private JPersistenceUnitInfoHelper() {

    }

    /**
     * Parses the given XML and complete the PersistenceUnitInfos structure
     * before returning it.
     * @param url the URL of the the Reader of the XML file.
     * @return array of jPersistenceInfo which are implementation of
     *         PersistenceUnitInfo object
     * @throws JPersistenceUnitInfoException if values are incorrect.
     */
    public static JPersistenceUnitInfo[] getPersistenceUnitInfo(final URL url) throws JPersistenceUnitInfoException {
        // load default values if not already loaded.
        loadDefaultValues();

        JPersistenceUnitInfo[] jPersistenceunitInfos = JPersistenceUnitInfoLoader.loadPersistenceUnitInfoImpl(url);

        for (JPersistenceUnitInfo jPersistenceunitInfo : jPersistenceunitInfos) {
            // analyze jta datasource
            String jtaDsName = jPersistenceunitInfo.getJtaDataSourceName();

            // try to get this datasource (if not null)
            if (jtaDsName != null && !jtaDsName.equals("")) {
                DataSource ds = null;
                try {
                    ds = (DataSource) new InitialContext().lookup(jtaDsName);
                } catch (NamingException e) {
                    // TODO: Remove
                    // try with jdbc_1
                    try {
                        logger.warn("Datasource named '" + jtaDsName
                                + "' was not found, use instead the default jndi name jdbc_1");
                        ds = (DataSource) new InitialContext().lookup("jdbc_1");
                    } catch (NamingException ne) {
                        throw new JPersistenceUnitInfoException("Cannot get jta DataSource with the JNDI name '"
                                + jtaDsName + "'.", ne);
                    }
                }
                jPersistenceunitInfo.setJtaDataSource(ds);
            }

            // analyze non jta datasource
            String nonJtaDsName = jPersistenceunitInfo.getNonJtaDataSourceName();
            // try to get this datasource (if not null)
            if (nonJtaDsName != null && !nonJtaDsName.equals("")) {
                DataSource ds = null;
                try {
                    ds = (DataSource) new InitialContext().lookup(nonJtaDsName);
                } catch (NamingException e) {
                    // TODO: Remove
                    // try with jdbc_1
                    try {
                        logger.warn("Datasource named '" + jtaDsName
                                + "' was not found, use instead the default jndi name jdbc_1");
                        ds = (DataSource) new InitialContext().lookup("jdbc_1");
                    } catch (NamingException ne) {
                        throw new JPersistenceUnitInfoException("Cannot get non jta DataSource with the JNDI name '"
                                + nonJtaDsName + "'.", ne);
                    }
                }
                jPersistenceunitInfo.setNonJtaDataSource(ds);
            }

            // Persistence Provider
            if (jPersistenceunitInfo.getPersistenceProviderClassName() == null
                    || jPersistenceunitInfo.getPersistenceProviderClassName().equals("")) {
                logger.info("No persistence provider was set, set to value {0}.", defaultPersistenceunitInfo
                        .getPersistenceProviderClassName());
                jPersistenceunitInfo.setPersistenceProviderClassName(defaultPersistenceunitInfo
                        .getPersistenceProviderClassName());
            }

            // add properties for the given persistence provider
            JPersistenceUnitInfo providerDefaultConf = providersInfo.get(jPersistenceunitInfo
                    .getPersistenceProviderClassName());
            if (providerDefaultConf == null) {
                logger.info("No default configuration for the persistence provider {0}", jPersistenceunitInfo
                        .getPersistenceProviderClassName());
            } else {
                logger.info("Found a default configuration for the persistence provider {0}", jPersistenceunitInfo
                        .getPersistenceProviderClassName());
                Properties defaultProperties = providerDefaultConf.getProperties();
                Enumeration providerPropertiesEnum = defaultProperties.propertyNames();
                while (providerPropertiesEnum.hasMoreElements()) {
                    String key = (String) providerPropertiesEnum.nextElement();
                    String value = defaultProperties.getProperty(key);
                    // set the value on the provider info
                    if (jPersistenceunitInfo.getProperties().getProperty(key) == null) {
                        jPersistenceunitInfo.getProperties().setProperty(key, value);
                        logger.info("Setting the property {0} with value {1}", key, value);
                    }
                }

            }
        }
        return jPersistenceunitInfos;
    }

    /**
     * Loads the default values (global configuration) and then configuration of
     * each persistence provider.
     * @throws JPersistenceUnitInfoException if default configuration is not found
     */
    private static synchronized void loadDefaultValues() throws JPersistenceUnitInfoException {

        // already loaded, return.
        if (defaultPersistenceunitInfo != null && providersInfo != null) {
            return;
        }
        defaultPersistenceunitInfo = new JPersistenceUnitInfo();
        providersInfo = new HashMap<String, JPersistenceUnitInfo>();

        // Load default values for a persistence INFO
        ClassLoader currentCL = Thread.currentThread().getContextClassLoader();
        Enumeration<URL> urlsConfig = null;
        try {
            urlsConfig = currentCL.getResources(DEFAULT_PERSISTENCE_CONFIG);
        } catch (IOException e) {
            throw new JPersistenceUnitInfoException("Cannot get resources with the name '" + DEFAULT_PERSISTENCE_CONFIG
                    + "' in the context classloader '" + currentCL + "'.");
        }

        // reverse the list of URLs as we want to override the values
        LinkedList<URL> lstURLs = new LinkedList<URL>();
        while (urlsConfig.hasMoreElements()) {
            lstURLs.addFirst(urlsConfig.nextElement());
        }

        // For each URL, analyze properties (only one persistence unit is
        // expected by module)
        for (URL tmpURL : lstURLs) {
            JPersistenceUnitInfo[] jPersistenceunitInfos = JPersistenceUnitInfoLoader
                    .loadPersistenceUnitInfoImpl(tmpURL);
            // use first unit name
            if (jPersistenceunitInfos.length != 1) {
                throw new JPersistenceUnitInfoException(
                        "Each default config file should have only one persistence unit '" + tmpURL + "'.");
            }
            JPersistenceUnitInfo pInfo = jPersistenceunitInfos[0];
            defaultPersistenceunitInfo.setPersistenceProviderClassName(pInfo.getPersistenceProviderClassName());

            // Now extract configuration of each persistence provider
            Properties providersProperties = pInfo.getProperties();
            Enumeration providerNames = providersProperties.propertyNames();
            while (providerNames.hasMoreElements()) {
                String providerName = (String) providerNames.nextElement();

                // check if there is an existing provider, else create it
                JPersistenceUnitInfo existingProviderInfo = providersInfo.get(providerName);
                if (existingProviderInfo == null) {
                    existingProviderInfo = new JPersistenceUnitInfo();
                    providersInfo.put(providerName, existingProviderInfo);
                }

                Enumeration<URL> urlsProviderConf = null;
                try {
                    urlsProviderConf = currentCL.getResources(providersProperties.getProperty(providerName));
                } catch (IOException e) {
                    throw new JPersistenceUnitInfoException("Cannot get resources with the name '" + providerName
                            + "' in the context classloader '" + currentCL + "'.");
                }
                // reverse order : used to override values
                LinkedList<URL> reverseProviderConfURLs = new LinkedList<URL>();
                while (urlsProviderConf.hasMoreElements()) {
                    reverseProviderConfURLs.addFirst(urlsProviderConf.nextElement());
                }

                if (reverseProviderConfURLs.size() == 0) {
                    logger.warn("No default properties for persistence provider class named {0}", providerName);
                }

                // for each info for a provider, set the values
                for (URL providerURLConf : reverseProviderConfURLs) {
                    JPersistenceUnitInfo[] providerPersistenceunitInfos = JPersistenceUnitInfoLoader
                            .loadPersistenceUnitInfoImpl(providerURLConf);
                    // use first unit name
                    if (providerPersistenceunitInfos.length != 1) {
                        throw new JPersistenceUnitInfoException(
                                "Each default config file should have only one persistence unit '" + providerURLConf
                                        + "'.");
                    }
                    JPersistenceUnitInfo providerInfo = providerPersistenceunitInfos[0];

                    // get provider info and set values
                    Properties providerProperties = providerInfo.getProperties();
                    Enumeration providerPropertiesEnum = providerProperties.propertyNames();
                    while (providerPropertiesEnum.hasMoreElements()) {
                        String key = (String) providerPropertiesEnum.nextElement();
                        String value = providerProperties.getProperty(key);
                        // set the value on the provider info
                        existingProviderInfo.getProperties().setProperty(key, value);
                    }

                }
            }

        }

        logger.info("Default persistence provider set to value {0}.", defaultPersistenceunitInfo
                .getPersistenceProviderClassName());
    }

}
