/**
 * EasyBeans
 * Copyright (C) 2006 Bull S.A.S.
 * Contact: easybeans@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: AbsInvocationHandler.java 630 2006-06-12 16:03:19Z benoitf $
 * --------------------------------------------------------------------------
 */

package org.objectweb.easybeans.rpc;

import java.io.Serializable;
import java.lang.reflect.InvocationHandler;
import java.lang.reflect.Method;
import java.util.HashMap;
import java.util.Map;

/**
 * Abstract class used by remote or local invocation handler.
 * @author Florent Benoit
 */
public abstract class AbsInvocationHandler implements InvocationHandler, Serializable {

    /**
     * Container id.
     */
    private String containerId = null;

    /**
     * Factory name.
     */
    private String factoryName = null;

    /**
     * Bean id.
     */
    private Long beanId = null;

    /**
     * Map between method and its hash.
     */
    private transient Map<Method, Long> hashedMethods = null;

    /**
     * Build a new Invocation handler.
     * @param containerId the id of the container that will be called on the
     *        remote side.
     * @param factoryName the name of the remote factory.
     */
    public AbsInvocationHandler(final String containerId, final String factoryName) {
        this.containerId = containerId;
        this.factoryName = factoryName;
        hashedMethods = new HashMap<Method, Long>();
    }

    /**
     * Manages all methods of java.lang.Object class.
     * @param method the <code>Method</code> instance corresponding to the
     *        interface method invoked on the proxy instance. The declaring
     *        class of the <code>Method</code> object will be the interface
     *        that the method was declared in, which may be a superinterface of
     *        the proxy interface that the proxy class inherits the method
     *        through.
     * @param args an array of objects containing the values of the arguments
     *        passed in the method invocation on the proxy instance
     * @return the value of the called method.
     */
    protected Object handleObjectMethods(final Method method, final Object[] args) {
        String methodName = method.getName();

        if (methodName.equals("equals")) {
            return Boolean.valueOf(this.toString().equals(args[0].toString()));
        } else if (methodName.equals("toString")) {
            return this.toString();
        } else if (methodName.equals("hashCode")) {
            return Integer.valueOf(this.toString().hashCode());
        } else {
            throw new IllegalStateException("Method '" + methodName + "' is not present on Object.class.");
        }
    }

    /**
     * Sets the id of the bean.
     * @param beanId the new ID.
     */
    protected void setBeanId(final Long beanId) {
        this.beanId = beanId;
    }

    /**
     * Sets the hashed methods.
     * @param hashedMethods the hash for each method.
     */
    protected void setHashedMethods(final Map<Method, Long> hashedMethods) {
        this.hashedMethods = hashedMethods;
    }

    /**
     * Gets the bean id.
     * @return the bean id.
     */
    protected Long getBeanId() {
        return beanId;
    }

    /**
     * @return the container id.
     */
    protected String getContainerId() {
        return containerId;
    }

    /**
     * @return the name of the factory.
     */
    protected String getFactoryName() {
        return factoryName;
    }

    /**
     * @return the hashes for each method.
     */
    protected Map<Method, Long> getHashedMethods() {
        return hashedMethods;
    }

    /**
     * Sets the container ID.
     * @param containerId the identifier of the container.
     */
    protected void setContainerId(final String containerId) {
        this.containerId = containerId;
    }

    /**
     * Sets the factory's name.
     * @param factoryName the name of the factory.
     */
    protected void setFactoryName(final String factoryName) {
        this.factoryName = factoryName;
    }

}
