/**
 * EasyBeans
 * Copyright (C) 2006 Bull S.A.S.
 * Contact: easybeans@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: LocalCallInvocationHandler.java 631 2006-06-12 16:05:39Z benoitf $
 * --------------------------------------------------------------------------
 */

package org.objectweb.easybeans.rpc;

import java.io.Externalizable;
import java.io.IOException;
import java.io.ObjectInput;
import java.io.ObjectOutput;
import java.lang.reflect.Method;
import java.util.HashMap;

import javax.ejb.EJBException;

import org.objectweb.easybeans.api.EZBContainer;
import org.objectweb.easybeans.api.EZBServer;
import org.objectweb.easybeans.api.Factory;
import org.objectweb.easybeans.rpc.api.EJBResponse;
import org.objectweb.easybeans.rpc.api.RPCException;
import org.objectweb.easybeans.rpc.util.Hash;
import org.objectweb.easybeans.server.EmbeddedManager;

/**
 * Object acting as the proxy for local interfaces calls.
 * @author Florent Benoit
 */
public class LocalCallInvocationHandler extends AbsInvocationHandler implements Externalizable {

    /**
     * UID for serialization.
     */
    private static final long serialVersionUID = -4327634481654235615L;

    /**
     * Embedded server ID.
     */
    private Integer embeddedID = null;

    /**
     * Factory for sending requests (build in constructor or when serialization
     * occurs).
     */
    private transient Factory factory = null;

    /**
     * Build a new Invocation handler.
     * @param ejb3server server on which send the calls
     * @param containerId the id of the container that will be called on the
     *        remote side.
     * @param factoryName the name of the remote factory.
     */
    public LocalCallInvocationHandler(final EZBServer ejb3server, final String containerId, final String factoryName) {
        super(containerId, factoryName);

        // Server ID
        embeddedID = ejb3server.getID();

        // Init the factory
        initFactory();
    }

    /**
     * Default constructor (used for serialization).
     */
    public LocalCallInvocationHandler() {
        super(null, null);
    }


    /**
     * Initialize the factory object with the given infos.
     */
    private void initFactory() {
        // Get Embedded server
        EZBServer ejb3Server = EmbeddedManager.getEmbedded(embeddedID);
        if (ejb3Server == null) {
            throw new IllegalStateException("Cannot find the server with id '" + embeddedID + "'.");
        }

        // Get the container
        EZBContainer container = ejb3Server.getContainer(getContainerId());
        if (container == null) {
            throw new IllegalStateException("Cannot find the container with id '" + getContainerId() + "'.");
        }

        factory = container.getFactory(getFactoryName());
        if (factory == null) {
            throw new IllegalStateException("Cannot find the factory with name '" + getFactoryName() + "'.");
        }
    }

    /**
     * Processes a method invocation on a proxy instance and returns the result.
     * This method will be invoked on an invocation handler when a method is
     * invoked on a proxy instance that it is associated with.
     * @param proxy the proxy instance that the method was invoked on
     * @param method the <code>Method</code> instance corresponding to the
     *        interface method invoked on the proxy instance. The declaring
     *        class of the <code>Method</code> object will be the interface
     *        that the method was declared in, which may be a superinterface of
     *        the proxy interface that the proxy class inherits the method
     *        through.
     * @param args an array of objects containing the values of the arguments
     *        passed in the method invocation on the proxy instance, or
     *        <code>null</code> if interface method takes no arguments.
     *        Arguments of primitive types are wrapped in instances of the
     *        appropriate primitive wrapper class, such as
     *        <code>java.lang.Integer</code> or <code>java.lang.Boolean</code>.
     * @return the value to return from the method invocation on the proxy
     *         instance.
     * @throws Exception the exception to throw from the method invocation on
     *         the proxy instance.
     */
    public Object invoke(final Object proxy, final Method method, final Object[] args) throws Exception {
        // Methods on the Object.class are not send on the remote side
        if (method.getDeclaringClass().getName().equals("java.lang.Object")) {
            return handleObjectMethods(method, args);
        }

        if (getHashedMethods() == null) {
            setHashedMethods(new HashMap<Method, Long>());
        }

        Long hashLong = getHashedMethods().get(method);
        if (hashLong == null) {
            hashLong = Long.valueOf(Hash.hashMethod(method));
            getHashedMethods().put(method, hashLong);
        }

        long hash = hashLong.longValue();

        // Now, need to invoke the bean
        EJBResponse response = null;
        try {
            response = factory.localCall(hash, args, getBeanId());
        } catch (RPCException e) {
            throw new EJBException("Error in invoking method on factory", (Exception) e.getCause());
        }
        setBeanId(response.getBeanId());
        return response.getValue();

    }

    /**
     * Save our content.
     * @param out the stream to write the object to
     * @throws IOException Includes any I/O exceptions that may occur
     */
    public void writeExternal(final ObjectOutput out) throws IOException {
        // server ID
        out.writeObject(embeddedID);
        // Container ID
        out.writeObject(getContainerId());
        // Factory name
        out.writeObject(getFactoryName());
    }

    /**
     * Build our content.
     * @param in the stream to read data from in order to restore the object
     * @exception IOException if I/O errors occur
     * @exception ClassNotFoundException If the class for an object being
     *            restored cannot be found.
     */
    public void readExternal(final ObjectInput in) throws IOException, ClassNotFoundException {
        // read ServerID
        this.embeddedID = (Integer) in.readObject();
        // read Container id
        setContainerId((String) in.readObject());
        // read factory's name
        setFactoryName((String) in.readObject());

        // init Factory object (transient)
        initFactory();

    }

}
