/**
 * EasyBeans
 * Copyright (C) 2006 Bull S.A.S.
 * Contact: easybeans@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: ContextHelper.java 434 2006-05-09 12:48:37Z studzine $
 * --------------------------------------------------------------------------
 */
package org.objectweb.easybeans.tests.common.helper;

import static org.objectweb.easybeans.tests.common.resources.EMFactoryTester.checkInstance;

import java.net.URL;

import javax.ejb.EJBContext;
import javax.ejb.SessionContext;
import javax.ejb.TimerService;
import javax.jms.ConnectionFactory;
import javax.jms.QueueConnectionFactory;
import javax.jms.TopicConnectionFactory;
import javax.mail.Session;
import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import javax.persistence.EntityManager;
import javax.persistence.EntityManagerFactory;
import javax.sql.DataSource;
import javax.transaction.UserTransaction;

import org.objectweb.easybeans.log.JLog;
import org.objectweb.easybeans.log.JLogFactory;
import org.objectweb.easybeans.tests.common.resources.EJBContextTester;
import org.objectweb.easybeans.tests.common.resources.EntityManagerTester;
import org.objectweb.easybeans.tests.common.resources.TimerServiceTester;
import org.objectweb.easybeans.tests.common.resources.UserTransactionTester;

/**
 * Used to do common operations on bean contexts.
 * @author Eduardo Studzinski Estima de Castro
 * @author Gisele Pinheiro Souza
 */
public final class ContextHelper {

    /**
     * Logger.
     */
    private static JLog logger = JLogFactory.getLog(ContextHelper.class);

    /**
     * Error message.
     */
    public static final String ERROR_MSG_INJECTION = "The container did not inject the default value specified "
            + "in the descriptor.";

    /**
     * Error message.
     */
    public static final String ERROR_MSG_NOT_FOUND = "Entry did not find in the environment.";

    /**
     * Error message.
     */
    public static final String ERROR_MSG_SESSION_CONTEXT = "Error in access using the Session Context.";

    /**
     * Error message.
     */
    public static final String ERROR_MSG_JNDI_ACCESS = "Error in access using the JNDI API directly.";

    /**
     * Error message.
     */
    public static final String ERROR_REFERENCE_NULL = "Reference is null.";

    /**
     * Creates a new instance of ContextHelper.
     *
     */
    private ContextHelper(){

    }

    /**
     * Checks the following items:<li>if a simple environment entry value specified
     * in the descriptor was correctly injected;</li> <li>if the session
     * context can be used to access the simple environment entry;</li> <li>if the
     * JNDI API can be used directly to access the simple environment entry;</li> If
     * fails, an IllegalStateException is thrown.
     * @param <E> Element Type
     * @param sessionContext session context instance
     * @param entryName entry name in the environment
     * @param beanValue value obtained in the bean object.
     * @param expectedValue value expected by the element
     */
    @SuppressWarnings("unchecked")
    public static <E> void checkSimpleEntry(final SessionContext sessionContext, final String entryName,
            final E beanValue, final E expectedValue) {

        // Injection
        logger.debug("Checking injection. Name = {0}", entryName);
        if (!expectedValue.equals(beanValue)) {
            throw new IllegalStateException(ERROR_MSG_INJECTION + " Name: " + entryName);
        }
        logger.debug("Injection is ok. Name = {0}", entryName);

        checkSimpleEntry(sessionContext, entryName, expectedValue);
    }

    /**
     * Checks the following items:<li>if the session context can be used to
     * access the environment entry;</li> <li>if the JNDI API can be used
     * directly to access the environment entry;</li> If fails, an
     * IllegalStateException is thrown.
     * @param <E> Element Type
     * @param sessionContext session context instance
     * @param entryName entry name in the environment
     * @param beanInterface bean interface.
     */
    @SuppressWarnings("unchecked")
    public static <E> void checkBeanRef(final SessionContext sessionContext, final String entryName,
            final E beanInterface) {

        logger.debug("Checking ejb reference. Name = {0}", entryName);

        //Session Context
        E sctxRef = (E) getEntryBySessionContext(sessionContext, entryName);
        checkBeanRef(sctxRef);

        //JNDI Access
        E jndiRef = (E) getEntryByJNDI(entryName);
        checkBeanRef(jndiRef);

        logger.debug("EJB reference is ok. Name = {0}", entryName);
    }

    /**
     * Gets an entry using the session context.
     * @param <E> entry type
     * @param sessionContext reference
     * @param entryName name in the enviroment
     * @return reference
     */
    @SuppressWarnings("unchecked")
    private static <E> E getEntryBySessionContext(final SessionContext sessionContext, final String entryName){
        //Check if the entry exists in the environment
        logger.debug("Getting reference using the session context. Name = {0}", entryName);

        E sctxEntry = (E) sessionContext.lookup(entryName);

        if (sctxEntry == null) {
            logger.debug("Entry reference is null. Name = {0} ", entryName);
        }

        logger.debug("Reference was gotten. Name = {0}", entryName);

        return sctxEntry;
    }

    /**
     * Gets an entry using the JNDI API.
     * @param <E> entry type
     * @param entryName name in the enviroment
     * @return reference
     */
    @SuppressWarnings("unchecked")
    private static <E> E getEntryByJNDI(final String entryName){
        //Check if the entry exists in the environment
        logger.debug("Getting reference using the JNDI API. Name = {0}", entryName);

        E eJNDI = null;
        try {
            Context initCtx = new InitialContext();
            Context myEnv = (Context) initCtx.lookup("java:comp/env");

            eJNDI = (E) myEnv.lookup(entryName);

            if (eJNDI == null) {
                logger.debug("Entry reference is null. Name = {0}", entryName);
            }
        } catch (NamingException e) {
            throw new IllegalStateException("The context could not be obtained or entry not found. Name = " + entryName);
        }

        logger.debug("Reference was gotten. Name = {0}", entryName);
        return eJNDI;
    }

    /**
     * Checks if a bean reference is not null.
     * @param <E> bean type
     * @param ref reference
     */
    private static <E> void checkBeanRef(final E ref){
        logger.debug("Checking ejb reference.");

        if (ref == null) {
            throw new IllegalStateException(ERROR_REFERENCE_NULL);
        }

        logger.debug("Ejb reference is ok.");
    }

    /**
     * Checks the following items:<li>if the session context can be used to
     * access the simple environment entry;</li> <li>if the JNDI API can be
     * used directly to access the simple environment entry;</li> If fails, an
     * IllegalStateException is thrown.
     * @param <E> Element Type
     * @param sessionContext session context instance
     * @param entryName entry name in the environment.
     * @param expectedValue value expected by the element.
     */
    @SuppressWarnings("unchecked")
    public static <E> void checkSimpleEntry(final SessionContext sessionContext, final String entryName,
            final E expectedValue) {

        logger.debug("Checking simple entry. Name = {0}", entryName);

        // Session Context
        E sctxValue = (E) getEntryBySessionContext(sessionContext, entryName);

        if (!expectedValue.equals(sctxValue)) {
            throw new IllegalStateException(ERROR_MSG_SESSION_CONTEXT + " Entry: " + entryName);
        }

        // JNDI Access
        E eJNDI = (E) getEntryByJNDI(entryName);

        if (!expectedValue.equals(eJNDI)) {
            throw new IllegalStateException(ERROR_MSG_JNDI_ACCESS);
        }

        logger.debug("Simple entry is ok. Name = {0}", entryName);
    }

    /**
     * Checks the following items:<li>if the resource was correctly injected;</li>
     * <li>if the session context can be used to
     * access the resource;</li> <li>if the JNDI API can be used
     * directly to access the resource;</li> If fails, an
     * IllegalStateException is thrown.
     * @param <E> Element Type
     * @param sessionContext session Context instance
     * @param resource resource object
     * @param resourceName resource name in the environment
     */
    @SuppressWarnings("unchecked")
    public static <E> void checkResource(final SessionContext sessionContext, final E resource, final String resourceName){
        //Injection
        checkResource(resource);
        //Environment access
        checkResource(sessionContext, resourceName);
    }

    /**
     * Checks the following items:<li>if the session context can be used to
     * access the entry;</li> <li>if the JNDI API can be used
     * directly to access the entry;</li> If fails, an
     * IllegalStateException is thrown.
     * @param <E> Element Type
     * @param sessionContext session context instance
     * @param entryName entry name in the environment
     */
    @SuppressWarnings("unchecked")
    public static <E> void checkResource(final SessionContext sessionContext, final String entryName) {

        logger.debug("Checking resource. Name = {0}", entryName);

        //Session Context
        E sctxEntry = (E) getEntryBySessionContext(sessionContext, entryName);
        checkResource(sctxEntry);

        //JNDI Access
        E eJNDI = (E) getEntryByJNDI(entryName);
        checkResource(eJNDI);

        logger.debug("Resource is ok. Name = {0}", entryName);
    }

    /**
     * Checks if an entry is working properly.
     * @param <E> entry type
     * @param entry reference
     */
    public static <E> void checkResource(final E entry) {
        if (entry == null) {
            throw new IllegalStateException(ERROR_REFERENCE_NULL);
        }

        String entryClassName = entry.getClass().getName();

        try {
            if (DataSource.class.getName().equals(entryClassName)) {
                // Checks DataSource
                logger.debug("Checking DataSource.");
                ((DataSource) entry).getConnection().close();

            } else if (ConnectionFactory.class.getName().equals(entryClassName)) {
                // Checks ConnectionFactory
                logger.debug("Checking ConnectionFactory.");
                ((ConnectionFactory) entry).createConnection().close();

            } else if (QueueConnectionFactory.class.getName().equals(entryClassName)) {
                // Checks QueueConnectionFactory
                logger.debug("Checking QueueConnectionFactory.");
                ((QueueConnectionFactory) entry).createConnection().close();

            } else if (TopicConnectionFactory.class.getName().equals(entryClassName)) {
                // Checks TopicConnectionFactory
                logger.debug("Checking TopicConnectionFactory.");
                ((TopicConnectionFactory) entry).createConnection().close();

            } else if (Session.class.getName().equals(entryClassName)) {
                // Checks Mail Session
                logger.debug("Checking Mail Session.");
                ((Session) entry).getProperties().keySet();

            } else if (URL.class.getName().equals(entryClassName)) {
                // Checks TopicConnectionFactory
                logger.debug("Checking URL.");
                ((URL) entry).getHost().toString();

            } else if (UserTransaction.class.getName().equals(entryClassName)) {
                // Checks UserTransaction
                logger.debug("Checking UserTransaction.");
                UserTransactionTester.checkInstance(((UserTransaction) entry));

            }else if (TimerService.class.getName().equals(entryClassName)) {
                // Checks TimerService
                logger.debug("Checking TimerService.");
                TimerServiceTester.checkInstance(((TimerService) entry));

            }else if (EJBContext.class.getName().equals(entryClassName)) {
                // Checks EJBContext
                logger.debug("Checking EJBContext.");
                EJBContextTester.checkInstance(((EJBContext) entry));

            }else if (SessionContext.class.getName().equals(entryClassName)) {
                // Checks SessionContext
                logger.debug("Checking EJBContext.");
                EJBContextTester.checkInstance(((SessionContext) entry));

            }else{
                logger.debug("Unknow resource. Type = {0}", entryClassName);
                throw new IllegalStateException("Unknow resource. Type = " + entryClassName);
            }
        } catch (Exception e) {
            logger.debug("Exception: {0}", e);
            throw new IllegalStateException("Exception: " + e.toString());
        }
    }

    /**
     * Checks the following items: <li>if the injected reference is ok;</li><li>if the session context can be used to
     * access the persitence unit entry;</li> <li>if the JNDI API can be used
     * directly to access the persistence unit entry;</li> If fails, an
     * IllegalStateException is thrown.
     * @param ref reference
     * @param sessionContext session context instance
     * @param pUnitName persistence unit name
     */
    public static void checkEntityManagerFactory(final SessionContext sessionContext, final EntityManagerFactory ref,
            final String pUnitName){
        logger.debug("Checking Entity Manager Factory. Name = {0}", pUnitName);

        //Check injection
        checkEntityManagerFactory(ref);

        //Environment
        checkEntityManagerFactory(sessionContext, pUnitName);

        logger.debug("Entity Manager Factory is ok. Name = {0}", pUnitName);
    }

    /**
     * Checks the following items:<li>if the session context can be used to
     * access the persitence unit entry;</li> <li>if the JNDI API can be used
     * directly to access the persistence unit entry;</li> If fails, an
     * IllegalStateException is thrown.
     * @param sessionContext session context instance
     * @param pUnitName persistence unit name
     */
    public static void checkEntityManagerFactory(final SessionContext sessionContext, final String pUnitName){
        logger.debug("Checking Entity Manager Factory. Name = {0}", pUnitName);

        // Session Context
        EntityManagerFactory sctxEntry = getEntryBySessionContext(sessionContext, pUnitName);
        checkEntityManagerFactory(sctxEntry);

        // JNDI Access
        EntityManagerFactory eJNDI = getEntryByJNDI(pUnitName);
        checkEntityManagerFactory(eJNDI);

        logger.debug("Entity Manager Factory is ok. Name = {0}", pUnitName);
    }

    /**
     * Checks if an entity manager factory reference is working properly.
     * @param ref reference
     */
    public static void checkEntityManagerFactory(final EntityManagerFactory ref){
        try{
            checkInstance(ref, "cemf");
        }catch(Exception e){
            throw new IllegalStateException("Error checking Entity Manager Factory reference.");
        }
    }

    /**
     * Checks the following items: <li>if the injected reference is ok;</li><li>if the session context can be used to
     * access the persitence unit entry;</li> <li>if the JNDI API can be used
     * directly to access the persistence context entry;</li> If fails, an
     * IllegalStateException is thrown.
     * @param ref reference
     * @param sessionContext session context instance
     * @param pUnitName persistence unit name
     */
    public static void checkEntityManager(final SessionContext sessionContext, final EntityManager ref,
            final String pUnitName){
        logger.debug("Checking Entity Manager Factory. Name = {0}", pUnitName);

        //Check injection
        checkEntityManager(ref);

        //Environment
        checkEntityManager(sessionContext, pUnitName);

        logger.debug("Entity Manager Factory is ok. Name = {0}", pUnitName);
    }

    /**
     * Checks the following items:<li>if the session context can be used to
     * access the persitence unit entry;</li> <li>if the JNDI API can be used
     * directly to access the persistence context entry;</li> If fails, an
     * IllegalStateException is thrown.
     * @param sessionContext session context instance
     * @param pUnitName persistence unit name
     */
    public static void checkEntityManager(final SessionContext sessionContext, final String pUnitName){
        logger.debug("Checking Entity Manager. Name = {0}", pUnitName);

        // Session Context
        EntityManager sctxEntry = getEntryBySessionContext(sessionContext, pUnitName);
        checkEntityManager(sctxEntry);

        // JNDI Access
        EntityManager eJNDI = getEntryByJNDI(pUnitName);
        checkEntityManager(eJNDI);

        logger.debug("Entity Manager is ok. Name = {0}", pUnitName);
    }

    /**
     * Checks if an entity manager reference is working properly.
     * @param ref reference
     */
    public static void checkEntityManager(final EntityManager ref){
        try{
            EntityManagerTester.checkInstance(ref, "cem");
        }catch(Exception e){
            throw new IllegalStateException("Error checking Entity Manager Factory reference.");
        }
    }

}
