/**
 * EasyBeans
 * Copyright (C) 2006 Bull S.A.S.
 * Contact: easybeans@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: TestMDBBasic00.java 692 2006-06-20 09:17:57Z studzine $
 * --------------------------------------------------------------------------
 */
package org.objectweb.easybeans.tests.messagedriven.containermanaged.basic;

import static org.objectweb.easybeans.tests.common.ejbs.entity.callbacklogger.CallbackType.ON_MESSAGE;
import static org.objectweb.easybeans.tests.common.ejbs.entity.callbacklogger.OperationType.UNDEFINED;
import static org.objectweb.easybeans.tests.common.helper.EJBHelper.getBeanRemoteInstance;

import java.util.ArrayList;
import java.util.List;

import org.objectweb.easybeans.tests.common.ejbs.mdb.containermanaged.basic.MDBQueue00;
import org.objectweb.easybeans.tests.common.ejbs.mdb.containermanaged.basic.MDBQueue01;
import org.objectweb.easybeans.tests.common.ejbs.mdb.containermanaged.basic.MDBTopic00;
import org.objectweb.easybeans.tests.common.ejbs.mdb.containermanaged.basic.MDBTopic01;
import org.objectweb.easybeans.tests.common.ejbs.stateless.containermanaged.callbacklogger.ItfCallbackLoggerAccess;
import org.objectweb.easybeans.tests.common.ejbs.stateless.containermanaged.callbacklogger.SLSBCallbackLoggerAccess;
import org.objectweb.easybeans.tests.common.jms.JMSManager;
import org.testng.annotations.Configuration;
import org.testng.annotations.Test;

/**
 * Verifies if simple message-driven beans can be deployed.
 * @reference JSR220-PROPOSED FINAL - Message-Driven Bean Component Contract
 * @requirement MDBs:<li>org.objectweb.easybeans.tests.common.ejbs.mdb.containermanaged.basic.MDBQueue00</li>
 *              <li>org.objectweb.easybeans.tests.common.ejbs.mdb.containermanaged.basic.MDBQueue01</li>
 *              <li>org.objectweb.easybeans.tests.common.ejbs.mdb.containermanaged.basic.MDBTopic00</li>
 *              <li>org.objectweb.easybeans.tests.common.ejbs.mdb.containermanaged.basic.MDBTopic01</li>
 * @author Eduardo Studzinski Estima de Castro
 * @author Gisele Pinheiro Souza
 */
public class TestMDBBasic00 {

    /**
     * JMS Manager.
     */
    private JMSManager jmsTopic;

    /**
     * JMS Manager.
     */
    private JMSManager jmsQueue;

    /**
     * Logger bean.
     */
    private ItfCallbackLoggerAccess beanLogger;

    /**
     * Creates the JMS manager.
     * @throws Exception if there is a problem.
     */
    @Configuration(beforeTestClass = true)
    public void startUp00() throws Exception {
        jmsQueue = new JMSManager(JMSManager.DEFAULT_QUEUE_CONNECTION_FACTORY, JMSManager.DEFAULT_QUEUE);
        jmsTopic = new JMSManager(JMSManager.DEFAULT_TOPIC_CONNECTION_FACTORY, JMSManager.DEFAULT_TOPIC);
        beanLogger = getBeanRemoteInstance(SLSBCallbackLoggerAccess.class, ItfCallbackLoggerAccess.class);
    }

    /**
     * Gets the bean logger instance and clears previous tests information.
     * @throws Exception if there is a problem with the bean initialization.
     */
    @Configuration(beforeTestMethod = true)
    public void startUp01() throws Exception {
        beanLogger.deleteAll();
    }

    /**
     * Verifies if a message-driven bean is invoked when a message is sent to a
     * topic.
     * @input a message
     * @output a logged onMessage() event
     * @throws Exception if a problem occurs.
     */
    @Test
    public void testTopic00() throws Exception {
        jmsTopic.sendControlMessage(MDBTopic00.MESSAGE_TYPE, UNDEFINED);

        // Verifies if the message was received
        List<String> arEvent = new ArrayList<String>();

        arEvent.add(MDBTopic00.class.getName());

        beanLogger.verifyCallbackOrder(MDBTopic00.class, ON_MESSAGE, arEvent.toArray(new String[0]));
    }

    /**
     * Verifies if a message-driven bean is invoked when a message is sent to a
     * topic.
     * @input a message
     * @output a logged onMessage() event
     * @throws Exception if a problem occurs.
     */
    @Test
    public void testTopic01() throws Exception {
        jmsTopic.sendControlMessage(MDBTopic01.MESSAGE_TYPE, UNDEFINED);

        // Verifies if the message was received
        List<String> arEvent = new ArrayList<String>();

        arEvent.add(MDBTopic01.class.getName());

        beanLogger.verifyCallbackOrder(MDBTopic01.class, ON_MESSAGE, arEvent.toArray(new String[0]));
    }

    /**
     * Verifies if a message-driven bean is invoked when a message is sent to a
     * queue.
     * @input a message
     * @output a logged onMessage() event
     * @throws Exception if a problem occurs.
     */
    @Test
    public void testQueue00() throws Exception {
        jmsQueue.sendControlMessage(MDBQueue00.MESSAGE_TYPE, UNDEFINED);

        // Verifies if the message was received
        List<String> arEvent = new ArrayList<String>();

        arEvent.add(MDBQueue00.class.getName());

        beanLogger.verifyCallbackOrder(MDBQueue00.class, ON_MESSAGE, arEvent.toArray(new String[0]));
    }

    /**
     * Verifies if a message-driven bean is invoked when a message is sent to a
     * queue.
     * @input a message
     * @output a logged onMessage() event
     * @throws Exception if a problem occurs.
     */
    @Test
    public void testQueue01() throws Exception {
        jmsQueue.sendControlMessage(MDBQueue01.MESSAGE_TYPE, UNDEFINED);

        // Verifies if the message was received
        List<String> arEvent = new ArrayList<String>();

        arEvent.add(MDBQueue01.class.getName());

        beanLogger.verifyCallbackOrder(MDBQueue01.class, ON_MESSAGE, arEvent.toArray(new String[0]));
    }

    /**
     * Clears logs.
     * @throws Exception if a problem occurs.
     */
    @Configuration(afterTestMethod = true)
    public void tearDown() throws Exception {
        beanLogger.deleteAll();
    }

    /**
     * Clears logs.
     * @throws Exception if a problem occurs.
     */
    @Configuration(afterTestClass = true)
    public void tearDownClass() throws Exception {
        jmsQueue.close();
        jmsTopic.close();
    }
}
